@@ -6,12 +6,10 @@ import sys
 import traceback
 from urllib.parse import urlparse
 
-from gi.repository import Gdk, Gtk, Gio
+from gi.repository import Gdk, Gio, Gtk
 
-from tryton import common
-from tryton import gui
-from tryton import translate
-from tryton.config import CONFIG, get_config_dir
+from tryton import __version__, common, gui, translate
+from tryton.config import CONFIG, copy_previous_configuration, get_config_dir
 from tryton.gui.window.dblogin import DBLogin
 
 
@@ -32,6 +30,9 @@ def main():
     label.editable {
         font-style: italic;
     }
+    label.warning {
+        color: @warning_color;
+    }
     .window-title, .wizard-title {
         font-size: large;
         font-weight: bold;
@@ -59,6 +60,9 @@ def main():
         common.error(value, ''.join(traceback.format_tb(traceback_)))
     sys.excepthook = excepthook
 
+    copy_previous_configuration('tryton.cfg')
+    copy_previous_configuration('profiles.cfg')
+    copy_previous_configuration('plugins')
     CONFIG.parse()
     if CONFIG.arguments:
         url = CONFIG.arguments[0]
@@ -80,7 +84,11 @@ def main():
             'database': CONFIG['login.db'],
             }
         server = hashlib.md5(server.encode('utf-8')).hexdigest()
-        application_id = 'org.tryton._' + server
+        version = ''.join(__version__.split('.')[:2])
+        application_id = 'org.tryton.Tryton-%(version)s._%(server)s' % {
+            'version': version,
+            'server': server,
+            }
         app = gui.Main(
             application_id=application_id,
             flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
