@@ -17,6 +17,8 @@ class NumberEntry(Gtk.Entry, Gtk.Editable):
     __digits = None
 
     def __init__(self, *args, **kwargs):
+        self.monetary = kwargs.pop('monetary', False)
+        self.convert = kwargs.pop('convert', float)
         super().__init__(*args, **kwargs)
         self.set_alignment(1.0)
         self.connect('key-press-event', self.__class__.__key_press_event)
@@ -35,14 +37,20 @@ class NumberEntry(Gtk.Entry, Gtk.Editable):
         text = self.get_text()
         if text:
             try:
-                return locale.atof(text)
+                return self.convert(locale.delocalize(text, self.monetary))
             except ValueError:
                 pass
         return None
 
     @property
     def __decimal_point(self):
-        return locale.localeconv()['decimal_point']
+        return locale.localeconv()[
+            self.monetary and 'mon_decimal_point' or 'decimal_point']
+
+    @property
+    def __thousands_sep(self):
+        return locale.localeconv()[
+            self.monetary and 'mon_thousands_sep' or 'thousands_sep']
 
     # XXX: Override vfunc because position is inout
     # https://gitlab.gnome.org/GNOME/pygobject/issues/12
@@ -51,20 +59,21 @@ class NumberEntry(Gtk.Entry, Gtk.Editable):
         text = self.get_buffer().get_text()
         text = text[:position] + new_text + text[position:]
         value = None
-        if text not in ['-', self.__decimal_point]:
+        if text not in ['-', self.__decimal_point, self.__thousands_sep]:
             try:
-                value = locale.atof(text)
-                if value.is_integer():
-                    value = int(value)
-                value = Decimal(str(value))
+                value = Decimal(locale.delocalize(text, self.monetary))
             except (ValueError, InvalidOperation):
                 return position
-        if (value and self.__digits is not None
-                and (round(value, self.__digits) != value
-                    or value.as_tuple().exponent < -self.__digits)):
+        try:
+            if (value and self.__digits is not None
+                    and (round(value, self.__digits) != value
+                        or value.as_tuple().exponent < -self.__digits)):
+                return position
+        except InvalidOperation:
             return position
-        buffer_.insert_text(position, new_text, len(new_text))
-        return position + len(new_text)
+        length = len(new_text.encode('utf-8'))
+        buffer_.insert_text(position, new_text, length)
+        return position + length
 
     def __key_press_event(self, event):
         for name in ['KP_Decimal', 'KP_Separator']:
