@@ -2,7 +2,6 @@
 # this repository contains the full copyright notices and license terms.
 "Form"
 import csv
-import datetime
 import gettext
 import locale
 import os
@@ -11,30 +10,30 @@ from itertools import zip_longest
 
 from gi.repository import Gdk, GLib, Gtk
 
-from tryton.gui.window.view_form.screen import Screen
+import tryton.common as common
+from tryton import plugins
 from tryton.action import Action
+from tryton.common import RPCException, RPCExecute, sur, sur_3b
+from tryton.common.common import selection as selection_
+from tryton.common.popup_menu import popup
+from tryton.common.underline import set_underline
 from tryton.gui import Main
 from tryton.gui.window import Window
-from tryton.gui.window.email_ import Email
-from tryton.gui.window.win_export import WinExport
-from tryton.gui.window.win_import import WinImport
 from tryton.gui.window.attachment import Attachment
+from tryton.gui.window.email_ import Email
+from tryton.gui.window.log import Log
 from tryton.gui.window.note import Note
 from tryton.gui.window.revision import Revision
-from tryton.signal_event import SignalEvent
-from tryton.common import message, sur, sur_3b, timezoned_date
-import tryton.common as common
-from tryton.common import RPCExecute, RPCException
-from tryton.common.common import selection as selection_
-from tryton.common.underline import set_underline
-from tryton import plugins
+from tryton.gui.window.view_form.screen import Screen
+from tryton.gui.window.win_export import WinExport
+from tryton.gui.window.win_import import WinImport
 
 from .tabcontent import TabContent
 
 _ = gettext.gettext
 
 
-class Form(SignalEvent, TabContent):
+class Form(TabContent):
     "Form"
 
     def __init__(self, model, res_id=None, name='', **attributes):
@@ -50,29 +49,26 @@ class Form(SignalEvent, TabContent):
             name = common.MODELNAME.get(model)
         self.name = name
 
+        loading_ids = res_id not in (None, False)
+        if loading_ids:
+            attributes.pop('tab_domain', None)
         self.screen = Screen(self.model, breadcrumb=[self.name], **attributes)
         self.screen.widget.show()
+        self.screen.windows.append(self)
 
         self.create_tabcontent()
 
         self.set_buttons_sensitive()
 
-        self.screen.signal_connect(self, 'record-message',
-            self._record_message)
-
-        self.screen.signal_connect(self, 'record-modified',
-            lambda *a: GLib.idle_add(self._record_modified, *a))
-        self.screen.signal_connect(self, 'record-saved', self._record_saved)
-        self.screen.signal_connect(
-            self, 'resources',
-            lambda screen, resources: self.update_resources(resources))
-
         self.attachment_screen = None
 
-        if res_id not in (None, False):
+        if loading_ids:
             if isinstance(res_id, int):
                 res_id = [res_id]
             self.screen.load(res_id)
+            if res_id:
+                self.screen.current_record = self.screen.group.get(res_id[0])
+            self.screen.display()
         else:
             if self.screen.current_view.view_type == 'form':
                 self.sig_new(None, autosave=False)
@@ -81,7 +77,7 @@ class Form(SignalEvent, TabContent):
                 self.screen.search_filter()
 
         self.update_revision()
-        self.activate_save()
+        self.set_buttons_sensitive()
 
     def get_toolbars(self):
         try:
@@ -127,8 +123,6 @@ class Form(SignalEvent, TabContent):
         return id(self)
 
     def destroy(self):
-        super(Form, self).destroy()
-        self.screen.signal_unconnect(self)
         self.screen.destroy()
 
     def sig_attach(self, widget=None):
@@ -223,13 +217,13 @@ class Form(SignalEvent, TabContent):
         but_prev.connect('clicked', lambda *a: screen.display_prev())
         but_next.connect('clicked', lambda *a: screen.display_next())
 
-        def update_label(screen, data):
-            position, length = data[:2]
-            label.set_text('(%s/%s)' % (position or '_', length))
-            but_prev.set_sensitive(position and position > 1)
-            but_next.set_sensitive(position and position < length)
+        class Preview():
+            def record_message(self, position, length, *args):
+                label.set_text('(%s/%s)' % (position or '_', length))
+                but_prev.set_sensitive(screen.has_prev())
+                but_next.set_sensitive(screen.has_next())
+        screen.windows.append(Preview())
 
-        screen.signal_connect(self, 'record-message', update_label)
         vbox.pack_start(screen.widget, expand=True, fill=True, padding=0)
         return vbox
 
@@ -247,6 +241,10 @@ class Form(SignalEvent, TabContent):
         if self.attachment_screen.domain != domain or force:
             self.attachment_screen.domain = domain
             self.attachment_screen.search_filter()
+            group = self.attachment_screen.group
+            if group:
+                self.attachment_screen.current_record = group[0]
+                self.attachment_screen.display()
 
     def sig_note(self, widget=None):
         record = self.screen.current_record
@@ -301,39 +299,10 @@ class Form(SignalEvent, TabContent):
     def sig_logs(self, widget=None):
         current_record = self.screen.current_record
         if not current_record or current_record.id < 0:
-            self.message_info(
+            self.info_bar_add(
                 _('You have to select one record.'), Gtk.MessageType.INFO)
-            return False
-
-        fields = [
-            ('id', _('ID:')),
-            ('create_uid.rec_name', _('Created by:')),
-            ('create_date', _('Created at:')),
-            ('write_uid.rec_name', _('Edited by:')),
-            ('write_date', _('Edited at:')),
-        ]
-
-        try:
-            data = RPCExecute('model', self.model, 'read', [current_record.id],
-                [x[0] for x in fields], context=self.screen.context)[0]
-        except RPCException:
             return
-        date_format = self.screen.context.get('date_format', '%x')
-        datetime_format = date_format + ' %H:%M:%S.%f'
-        message_str = ''
-        for (key, label) in fields:
-            value = data
-            keys = key.split('.')
-            name = keys.pop(-1)
-            for key in keys:
-                value = value.get(key + '.', {})
-            value = (value or {}).get(name, '/')
-            if isinstance(value, datetime.datetime):
-                value = timezoned_date(value).strftime(datetime_format)
-            message_str += '%s %s\n' % (label, value)
-        message_str += _('Model:') + ' ' + self.model
-        message(message_str)
-        return True
+        Log(current_record)
 
     def sig_revision(self, widget=None):
         if not self.modified_save():
@@ -382,14 +351,18 @@ class Form(SignalEvent, TabContent):
             tooltip = self.name
         self.title.set_text(label)
         tooltips.set_tip(self.title, tooltip)
-        self.set_buttons_sensitive(revision)
+        self.set_buttons_sensitive()
 
-    def set_buttons_sensitive(self, revision=None):
+    def set_buttons_sensitive(self):
+        revision = self.screen.context.get('_datetime')
         if not revision:
             access = common.MODELACCESS[self.model]
+            modified = self.screen.modified()
             for name, sensitive in [
-                    ('new', access['create']),
-                    ('save', access['create'] or access['write']),
+                    ('new', access['create'] and not modified),
+                    ('save',
+                        (access['create'] or access['write'])
+                        and modified and not self.screen.readonly),
                     ('remove', access['delete']),
                     ('copy', access['create']),
                     ('import', access['create']),
@@ -415,10 +388,10 @@ class Form(SignalEvent, TabContent):
             msg = _('Are you sure to remove those records?')
         if sur(msg):
             if not self.screen.remove(delete=True, force_remove=True):
-                self.message_info(
+                self.info_bar_add(
                     _('Records not removed.'), Gtk.MessageType.ERROR)
             else:
-                self.message_info(_('Records removed.'), Gtk.MessageType.INFO)
+                self.info_bar_add(_('Records removed.'), Gtk.MessageType.INFO)
                 self.screen.count_tab_domain(True)
 
     def sig_import(self, widget=None):
@@ -441,9 +414,7 @@ class Form(SignalEvent, TabContent):
     def do_export(self, widget, export):
         if not self.modified_save():
             return
-        if (self.screen.current_view
-                and self.screen.current_view.view_type == 'tree'
-                and self.screen.current_view.children_field):
+        if export.get('records') == 'listed':
             ids = [r.id for r in self.screen.listed_records]
             paths = self.screen.listed_paths
         else:
@@ -451,7 +422,7 @@ class Form(SignalEvent, TabContent):
             paths = self.screen.selected_paths
         fields = [f['name'] for f in export['export_fields.']]
         data = RPCExecute(
-            'model', self.model, 'export_data', ids, fields,
+            'model', self.model, 'export_data', ids, fields, export['header'],
             context=self.screen.context)
         delimiter = ','
         if os.name == 'nt' and ',' == locale.localeconv()['decimal_point']:
@@ -460,7 +431,6 @@ class Form(SignalEvent, TabContent):
             '.csv', common.slugify(export['name']) + '_')
         with open(fname, 'w') as fp:
             writer = csv.writer(fp, delimiter=delimiter)
-            writer.writerow(fields)
             for row, path in zip_longest(data, paths or []):
                 indent = len(path) - 1 if path else 0
                 if row:
@@ -475,8 +445,8 @@ class Form(SignalEvent, TabContent):
             if not self.modified_save():
                 return
         self.screen.new()
-        self.message_info()
-        self.activate_save()
+        self.info_bar_clear()
+        self.set_buttons_sensitive()
 
     def sig_copy(self, widget=None):
         if not common.MODELACCESS[self.model]['create']:
@@ -484,7 +454,8 @@ class Form(SignalEvent, TabContent):
         if not self.modified_save():
             return
         if self.screen.copy():
-            self.message_info(_('Working now on the duplicated record(s).'),
+            self.info_bar_add(
+                _('Working now on the duplicated record(s).'),
                 Gtk.MessageType.INFO)
             self.screen.count_tab_domain(True)
 
@@ -497,11 +468,11 @@ class Form(SignalEvent, TabContent):
                 or self.screen.writable):
             return
         if self.screen.save_current():
-            self.message_info(_('Record saved.'), Gtk.MessageType.INFO)
+            self.info_bar_add(_('Record saved.'), Gtk.MessageType.INFO)
             self.screen.count_tab_domain(True)
             return True
         else:
-            self.message_info(
+            self.info_bar_add(
                 self.screen.invalid_message(), Gtk.MessageType.ERROR)
             return False
 
@@ -509,15 +480,15 @@ class Form(SignalEvent, TabContent):
         if not self.modified_save():
             return
         self.screen.display_prev()
-        self.message_info()
-        self.activate_save()
+        self.info_bar_clear()
+        self.set_buttons_sensitive()
 
     def sig_next(self, widget=None):
         if not self.modified_save():
             return
         self.screen.display_next()
-        self.message_info()
-        self.activate_save()
+        self.info_bar_clear()
+        self.set_buttons_sensitive()
 
     def sig_reload(self, test_modified=True):
         if test_modified:
@@ -537,9 +508,8 @@ class Form(SignalEvent, TabContent):
                     set_cursor = True
                     break
         self.screen.display(set_cursor=set_cursor)
-        self.message_info()
+        self.info_bar_clear()
         self.set_buttons_sensitive()
-        self.activate_save()
         self.screen.count_tab_domain()
         return True
 
@@ -596,28 +566,22 @@ class Form(SignalEvent, TabContent):
         if not widget.props.active:
             menu.popdown()
             return
+        popup(menu, widget)
 
-        def menu_position(menu, x, y, user_data):
-            widget_allocation = widget.get_allocation()
-            x, y = widget.get_window().get_root_coords(
-                widget_allocation.x, widget_allocation.y)
-            return (x, y + widget_allocation.height, False)
-        menu.show_all()
-        if hasattr(menu, 'popup_at_widget'):
-            menu.popup_at_widget(
-                widget, Gdk.Gravity.SOUTH_WEST, Gdk.Gravity.NORTH_WEST,
-                Gtk.get_current_event())
-        else:
-            menu.popup(
-                None, None, menu_position, None, 0,
-                Gtk.get_current_event_time())
+    def record_message(self, position, size, max_size, record_id):
+        def set_sensitive(button_id, sensitive):
+            if button_id in self.buttons:
+                self.buttons[button_id].props.sensitive = sensitive
+            if button_id in self.menu_buttons:
+                self.menu_buttons[button_id].props.sensitive = sensitive
 
-    def _record_message(self, screen, signal_data):
-        name = '_'
-        if signal_data[0]:
-            name = str(signal_data[0])
-        for button_id in ('print', 'relate', 'email', 'open', 'save',
-                'attach'):
+        name = str(position) if position else '_'
+        selected = len(self.screen.selected_records)
+        view_type = self.screen.current_view.view_type
+        has_views = self.screen.number_of_views > 1
+        if selected > 1:
+            name += '#%i' % selected
+        for button_id in ['print', 'relate', 'email', 'open', 'attach']:
             button = self.buttons[button_id]
             can_be_sensitive = getattr(button, '_can_be_sensitive', True)
             if button_id in {'print', 'relate', 'email', 'open'}:
@@ -626,37 +590,37 @@ class Form(SignalEvent, TabContent):
                     action_type = 'print'
                 can_be_sensitive |= any(
                     b.attrs.get('keyword', 'action') == action_type
-                    for b in screen.get_buttons())
-            elif button_id == 'save':
-                can_be_sensitive &= not self.screen.readonly
-            button.props.sensitive = (bool(signal_data[0])
-                and can_be_sensitive)
-        button_switch = self.buttons['switch']
-        button_switch.props.sensitive = self.screen.number_of_views > 1
+                    for b in self.screen.get_buttons())
+            set_sensitive(button_id, bool(position) and can_be_sensitive)
+        set_sensitive(
+            'switch', (position or view_type == 'form') and has_views)
+        set_sensitive('remove', self.screen.deletable)
+        set_sensitive('previous', self.screen.has_prev())
+        set_sensitive('next', self.screen.has_next())
 
-        menu_delete = self.menu_buttons['remove']
-        menu_delete.props.sensitive = self.screen.deletable
-        menu_save = self.menu_buttons['save']
-        menu_save.props.sensitive = not self.screen.readonly
-
-        msg = name + ' / ' + str(signal_data[1])
-        if (signal_data[1] < signal_data[2]
-                and self.screen.limit is not None
-                and signal_data[2] > self.screen.limit):
-            msg += _(' of ') + str(signal_data[2])
+        if size < max_size:
+            msg = "%s@%s/%s" % (
+                name, common.humanize(size), common.humanize(max_size))
+            if max_size >= self.screen.count_limit:
+                msg += "+"
+        else:
+            msg = "%s/%s" % (name, common.humanize(size))
         self.status_label.set_text(msg)
-        self.message_info()
-        self.activate_save()
+        self.info_bar_clear()
+        self.set_buttons_sensitive()
         self.refresh_attachment_preview()
 
-    def _record_modified(self, screen, signal_data):
-        # As it is called via idle_add, the form could have been destroyed in
-        # the meantime.
-        if self.widget_get().props.window:
-            self.activate_save()
+    def record_modified(self):
+        def _record_modified():
+            # As it is called via idle_add, the form could have been destroyed
+            # in the meantime.
+            if self.widget_get().props.window:
+                self.set_buttons_sensitive()
+        GLib.idle_add(_record_modified)
+        self.info_bar_refresh()
 
-    def _record_saved(self, screen, signal_data):
-        self.activate_save()
+    def record_saved(self):
+        self.set_buttons_sensitive()
         self.refresh_resources()
 
     def modified_save(self):
@@ -707,9 +671,6 @@ class Form(SignalEvent, TabContent):
         }
         Action.execute(action, data, context=self.screen.local_context)
 
-    def activate_save(self):
-        self.buttons['save'].props.sensitive = self.screen.modified()
-
     def sig_win_close(self, widget):
         Main().sig_win_close(widget)
 
