@@ -1,3 +1,5 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import screen
+from . import screen
+
+__all__ = [screen]
