@@ -2,14 +2,14 @@
 # this repository contains the full copyright notices and license terms.
 import gettext
 import os
-from urllib.request import urlopen
-from urllib.parse import urlparse, unquote
+from urllib.parse import unquote, urlparse
 
 from gi.repository import Gdk, Gtk
 
-from tryton.common import common
-from tryton.common import file_selection, Tooltips, file_open, file_write
+from tryton.common import (
+    Tooltips, common, file_open, file_selection, file_write, url_open)
 from tryton.common.entry_position import reset_position
+
 from .widget import Widget
 
 _ = gettext.gettext
@@ -28,7 +28,7 @@ class BinaryMixin(Widget):
 
         self.but_save_as = Gtk.Button()
         self.but_save_as.set_image(common.IconFactory.get_image(
-                'tryton-save', Gtk.IconSize.SMALL_TOOLBAR))
+                'tryton-download', Gtk.IconSize.SMALL_TOOLBAR))
         self.but_save_as.set_relief(Gtk.ReliefStyle.NONE)
         self.but_save_as.connect('clicked', self.save_as)
         tooltips.set_tip(self.but_save_as, _('Save As...'))
@@ -91,7 +91,7 @@ class BinaryMixin(Widget):
         filename = file_selection(
             _('Select'), preview=self.preview, filters=self.filters)
         if filename:
-            self._set_uri('file:///' + filename)
+            self._set_uri(filename.as_uri())
 
     def select_drag_data_received(
             self, widget, context, x, y, selection, info, timestamp):
@@ -102,7 +102,7 @@ class BinaryMixin(Widget):
 
     def _set_uri(self, uri):
         uri = unquote(uri)
-        self.field.set_client(self.record, urlopen(uri).read())
+        self.field.set_client(self.record, url_open(uri).read())
         if self.filename_field:
             self.filename_field.set_client(self.record,
                 os.path.basename(urlparse(uri).path))
@@ -209,7 +209,7 @@ class Binary(BinaryMixin, Widget):
             size = self.field.get_size(self.record)
         else:
             size = len(self.field.get(self.record))
-        self.wid_size.set_text(common.humanize(size or 0))
+        self.wid_size.set_text(common.humanize(size or 0, 'B'))
         reset_position(self.wid_size)
         if self.wid_text:
             self.wid_text.set_text(self.filename_field.get(self.record) or '')
