@@ -4,11 +4,12 @@ import gettext
 
 from gi.repository import GLib, Gtk
 
-from .widget import Widget, TranslateMixin
-from tryton.common import Tooltips, IconFactory
+from tryton.common import IconFactory, Tooltips
 from tryton.common.entry_position import reset_position
 from tryton.common.selection import PopdownMixin, selection_shortcuts
 
+from .widget import TranslateMixin, Widget
+
 _ = gettext.gettext
 
 
