@@ -1,9 +1,11 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
-from .widget import Widget
 import gettext
 
+from gi.repository import Gtk
+
+from .widget import Widget
+
 _ = gettext.gettext
 
 
@@ -11,7 +13,7 @@ class CheckBox(Widget):
 
     def __init__(self, view, attrs):
         super(CheckBox, self).__init__(view, attrs)
-        self.widget = self.mnemonic_widget = gtk.CheckButton()
+        self.widget = self.mnemonic_widget = Gtk.CheckButton()
         self.widget.connect('focus-out-event', lambda x, y: self._focus_out())
         self.widget.connect_after('toggled', self.sig_activate)
 
@@ -20,12 +22,16 @@ class CheckBox(Widget):
         # TODO find a better solution to accept focus
         self.widget.set_sensitive(not value)
 
-    def set_value(self, record, field):
-        field.set_client(record, self.widget.get_active())
+    def set_value(self):
+        self.field.set_client(self.record, self.widget.get_active())
 
-    def display(self, record, field):
-        super(CheckBox, self).display(record, field)
-        if not field:
+    def display(self):
+        super(CheckBox, self).display()
+        if not self.field:
             self.widget.set_active(False)
             return False
-        self.widget.set_active(bool(field.get(record)))
+        self.widget.handler_block_by_func(self.sig_activate)
+        try:
+            self.widget.set_active(bool(self.field.get(self.record)))
+        finally:
+            self.widget.handler_unblock_by_func(self.sig_activate)
